Class: Conductor::Worker::Events::TaskRunnerEvent
- Inherits:
-
ConductorEvent
- Object
- ConductorEvent
- Conductor::Worker::Events::TaskRunnerEvent
- Defined in:
- lib/conductor/worker/events/conductor_event.rb
Overview
Base class for task runner events All events related to task polling and execution inherit from this
Direct Known Subclasses
ActiveWorkersChanged, PollCompleted, PollFailure, PollStarted, TaskExecutionCompleted, TaskExecutionFailure, TaskExecutionStarted, TaskPaused, TaskUpdateCompleted, TaskUpdateFailure
Instance Attribute Summary collapse
-
#task_type ⇒ String
readonly
The task type/definition name.
Attributes inherited from ConductorEvent
Instance Method Summary collapse
-
#initialize(task_type:) ⇒ TaskRunnerEvent
constructor
A new instance of TaskRunnerEvent.
- #to_h ⇒ Object
Constructor Details
#initialize(task_type:) ⇒ TaskRunnerEvent
Returns a new instance of TaskRunnerEvent.
29 30 31 32 |
# File 'lib/conductor/worker/events/conductor_event.rb', line 29 def initialize(task_type:) super() @task_type = task_type end |
Instance Attribute Details
#task_type ⇒ String (readonly)
Returns The task type/definition name.
26 27 28 |
# File 'lib/conductor/worker/events/conductor_event.rb', line 26 def task_type @task_type end |
Instance Method Details
#to_h ⇒ Object
34 35 36 |
# File 'lib/conductor/worker/events/conductor_event.rb', line 34 def to_h super.merge(task_type: @task_type) end |